from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .....UtilityAndView.abaqusConstants import OFF, Boolean
from .CapCreepCohesion import CapCreepCohesion
from .CapCreepConsolidation import CapCreepConsolidation
from .CapHardening import CapHardening


@abaqus_class_doc
class CapPlasticity:
    r"""The CapPlasticity object specifies the modified Drucker-Prager/Cap plasticity model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].capPlasticity
            import odbMaterial
            session.odbs[name].materials[name].capPlasticity

        The table data for this object are:

        - Material cohesion, :math:`d`, in the :math:`p-t` plane (Abaqus/Standard) or in the :math:`p-q`
          plane (Abaqus/Explicit).
        - Material angle of friction, :math:`\beta`, in the :math:`p-t` plane (Abaqus/Standard) or
          in the :math:`p-q` plane (Abaqus/Explicit). Give the value in degrees.
        - Cap eccentricity parameter, :math:`R`. Its value must be greater than zero (typically
          :math:`0.0<R<1.0)`.
        - Initial cap yield surface position, :math:`\left.\varepsilon_{v o l}^{p l}\right|_{0}`
        - Transition surface radius parameter, :math:`\alpha`. The default value is :math:`0.0` (i.e.,
          no transition surface). Abaqus/Standard assumes :math:`K=1.0`.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - CAP PLASTICITY
    """

    #: A CapCreepCohesion object.
    capCreepCohesion: CapCreepCohesion = CapCreepCohesion(((),))

    #: A CapCreepConsolidation object.
    capCreepConsolidation: CapCreepConsolidation = CapCreepConsolidation(((),))

    #: A CapHardening object.
    capHardening: CapHardening = CapHardening(((),))

    @abaqus_method_doc
    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a CapPlasticity object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].CapPlasticity
                session.odbs[name].materials[name].CapPlasticity

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        CapPlasticity
            A CapPlasticity object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the CapPlasticity object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from .FXComposite import FXComposite


class FXShell(FXComposite):
    """A child of the Root window."""

    def create(self):
        """Create server-side resources.

        Reimplemented from FXComposite. Reimplemented in FXPrintDialog, FXToolbarShell, FXTooltip,
        FXTopWindow, AFXManagerMenuPane, AFXMainWindow, and AFXDialog.
        """

    def recalc(self):
        """Mark this window's layout as dirty.

        Reimplemented from FXWindow.
        """



import re

from setuptools import setup


def get_abqpy_version():
    try:
        import setuptools_scm

        version = setuptools_scm.get_version(root="..", version_scheme="post-release")
        major, minor, *_ = re.match(r"(\d+)\.(\d+)(.*)", version).groups()
        return f"{major}.{minor}"
    except (LookupError, ImportError):
        return "2020.*"


setup(install_requires=[f"abqpy=={get_abqpy_version()}"])



from __future__ import annotations

import copy

from typing_extensions import Literal, overload

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Job.JobMdb import JobMdb
from ..Model.Model import Model
from ..Part.AcisMdb import AcisMdb
from ..UtilityAndView.abaqusConstants import (
    B31,
    C3D8I,
    C3D10,
    NOT_SET,
    OFF,
    ON,
    PRESERVE_SECTION,
    S4,
    STANDARD_EXPLICIT,
    Boolean,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Mdb(AcisMdb, JobMdb):
    """The Mdb object is the high-level Abaqus model database. A model database stores models and analysis
    controls.

    .. note::
        This object can be accessed by::

            mdb
    """

    @overload
    @abaqus_method_doc
    def Model(
        self,
        name: str,
        description: str = "",
        stefanBoltzmann: float | None = None,
        absoluteZero: float | None = None,
        waveFormulation: Literal[C.SCATTERED, C.NOT_SET, C.TOTAL] = NOT_SET,
        modelType: Literal[C.STANDARD_EXPLICIT, C.ELECTROMAGNETIC] = STANDARD_EXPLICIT,
        universalGas: float | None = None,
        copyConstraints: Boolean = ON,
        copyConnectors: Boolean = ON,
        copyInteractions: Boolean = ON,
    ) -> Model:
        """This method creates a Model object.

        .. note::
            This function can be accessed by::

                mdb.Model

        Parameters
        ----------
        name
            A String specifying the repository key.
        description
            A String specifying the purpose and contents of the Model object. The default value is
            an empty string.
        stefanBoltzmann
            None or a Float specifying the Stefan-Boltzmann constant. The default value is None.
        absoluteZero
            None or a Float specifying the absolute zero constant. The default value is None.
        waveFormulation
            A SymbolicConstant specifying the type of incident wave formulation to be used in
            acoustic problems. Possible values are NOT_SET, SCATTERED, and TOTAL. The default value
            is NOT_SET.
        modelType
            A SymbolicConstant specifying the analysis model type. Possible values are
            STANDARD_EXPLICIT and ELECTROMAGNETIC. The default is STANDARD_EXPLICIT.
        universalGas
            None or a Float specifying the universal gas constant. The default value is None.
        copyConstraints
            A boolean specifying whether to copy the constraints created in the model to the model
            that instances this model. The default value is ON.
        copyConnectors
            A boolean specifying whether to copy the connectors created in the model to the model
            that instances this model. The default value is ON.
        copyInteractions
            A boolean specifying whether to copy the interactions created in the model to the model
            that instances this model. The default value is ON.

        Returns
        -------
        model: Model
            A Model object
        """

    @overload
    @abaqus_method_doc
    def Model(self, name: str, objectToCopy: Model) -> Model:
        """This method creates a Model object.

        .. note::
            This function can be accessed by::

                mdb.Model

        Parameters
        ----------
        name
            A String specifying the repository key.
        objectToCopy
            A Model object to copy.
        """

    @abaqus_method_doc
    def Model(self, name: str, *args, **kwargs) -> Model:
        """This method creates a Model object.

        .. note::
            This function can be accessed by::

                mdb.Model

        Parameters
        ----------
        name
            A String specifying the repository key.
        args, kwargs
            Positional and keyword arguments to be passed to the Model object.
        """
        if len(args) == 1 and isinstance(args[0], Model):
            self.models[name] = model = copy.deepcopy(args[0])
        elif "objectToCopy" in kwargs and isinstance(kwargs["objectToCopy"], Model):
            self.models[name] = model = copy.deepcopy(kwargs["objectToCopy"])
        else:
            self.models[name] = model = Model(name, *args, **kwargs)
        return model

    @abaqus_method_doc
    def ModelFromInputFile(self, name: str, inputFileName: str):
        """This method creates a Model object by reading the keywords in an input file and creating the
        corresponding Abaqus/CAE objects.

        .. note::
            This function can be accessed by::

                mdb.ModelFromInputFile

        Parameters
        ----------
        name
            A String specifying the repository key.
        inputFileName
            A String specifying the name of the input file (including the .inp extension) to be
            parsed into the new model. This String can also be the full path to the input file if it
            is located in another directory.

        Returns
        -------
        Model
            A Model object.
        """
        self.models[name] = model = Model(name)
        return model

    @abaqus_method_doc
    def ModelFromOdbFile(self, name: str, odbFileName: str):
        """This method creates a Model object by reading an output database and creating any corresponding
        Abaqus/CAE objects.

        .. note::
            This function can be accessed by::

                mdb.ModelFromOdbFile

        Parameters
        ----------
        name
            A String specifying the repository key.
        odbFileName
            A String specifying the name of the output database file (including the .odb extension)
            to be read into the new model. This String can also be the full path to the output
            database file if it is located in another directory.

        Returns
        -------
        Model
            A Model object.
        """
        self.models[name] = model = Model(name)
        return model

    @abaqus_method_doc
    def ModelFromNastranFile(
        self,
        modelName: str,
        inputFileName: str,
        sectionConsolidation: Literal[C.PRESERVE_SECTION, C.GROUP_BY_MATERIAL, C.NONE] = PRESERVE_SECTION,
        preIntegratedShell: Boolean = OFF,
        weightMassScaling: Boolean = ON,
        loadCases: Boolean = ON,
        coupleBeamOffsets: Boolean = ON,
        cbar: str = B31,
        cquad4: str = S4,
        chexa: str = C3D8I,
        ctetra: str = C3D10,
        keepTranslatedFiles: Boolean = ON,
    ):
        """This method creates a Model object by reading the keywords in a Nastran bulk data file or Nastran
        input file and creating any corresponding Abaqus/CAE objects. The default values is discussed in
        following and can be defined alternatively in the Abaqus environment file as the one used for the
        translator from Nastran to Abaqus. For more information, see Translating Nastran data to Abaqus files.

        .. note::
            This function can be accessed by::

                mdb.ModelFromNastranFile

        Parameters
        ----------
        modelName
            A String specifying the repository key.
        inputFileName
            A String specifying the name of the Nastran input file (including the .bdf, .dat, .nas,
            .nastran, .blk, .bulk extension) to be read into the new model. This String can also be
            the full path to the Nastran input file if it is located in another directory.
        sectionConsolidation
            A SymbolicConstant specifying the method used to create shell section. Possible values
            are PRESERVE_SECTION, GROUP_BY_MATERIAL, and NONE. If PRESERVE_SECTION is used, an
            Abaqus section is created corresponding to each shell property ID. If GROUP_BY_MATERIAL
            is used, a single Abaqus section is created for all homogeneous elements referencing the
            same material. In both cases, material orientations and offsets are created using
            discrete fields. If NONE is used, a separate shell section is created for each
            combination of orientation, material offset, and/or thickness. The default is
            PRESERVE_SECTION.
        preIntegratedShell
            A Boolean specifying whether the pre-integrated shell section is created in default for
            shell element. The default value is OFF.
        weightMassScaling
            A Boolean specifying whether the value on the Nastran data line PARAM, WTMASS is used as
            a multiplier for all density, mass, and rotary inertia values created in the Abaqus
            input file. The default value is ON.
        loadCases
            A Boolean specifying whether each SUBCASE for linear static analyses is translated to a
            LOAD CASE option, and all such LOAD CASE options are grouped in a single STEP option.
            The default value is ON.
        coupleBeamOffsets
            A Boolean specifying whether to translate the beam element connectivity to newly created
            nodes at the offset location and rigidly coupling the new and original nodes. If not,
            beam element offsets are translated to the CENTROID and SHEAR CENTER options, which are
            suboptions of the BEAM GENERAL SECTION option. The default value is ON. When the beam
            element references a PBARL or PBEAML property or if the beam offset has a significant
            component in the direction of the beam axis, the setting for this argument is always ON.
        cbar
            A String specifying the 2-node beam that is created from CBAR and CBEAM elements.
            Possible values are B31 and B33. The default is B31.
        cquad4
            A String specifying the 4-node shell that is created from CQUAD4 elements. Possible
            values are S4 and S4R. The default is S4. If a reduced-integration element is chosen,
            the enhanced hourglass formulation is applied automatically.
        chexa
            A String specifying the 8-node brick that is created from CHEXA elements. Possible
            values are C3D8I, C3D8 and C3D8R. The default is C3D8I. If a reduced-integration element
            is chosen, the enhanced hourglass formulation is applied automatically.
        ctetra
            A String specifying the 10-node tetrahedron that is created from CTETRA elements.
            Possible values are C3D10 and C3D10M. The default is C3D10.
        keepTranslatedFiles
            A Boolean specifying whether to keep the generated Abaqus input file after the model is
            created from the Nastran input file. The default value is ON.

        Returns
        -------
        Model
            A Model object.
        """
        self.models[modelName] = model = Model(modelName)
        return model



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..PlotOptions.OdbDisplayOptions import OdbDisplayOptions
from ..UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class DisplayGroupInstance:
    """A DisplayGroupInstance object stores the IDs of the entities displayed in a viewport. The
    DisplayGroupInstance object has no constructor. When you set a display group to be plotted in a viewport,
    Abaqus/CAE creates a DisplayGroupInstance object for each display group and places it in the
    DisplayGroupInstanceRepository object.

    .. note::
        This object can be accessed by::

            import assembly
            session.viewports[name].assemblyDisplay.displayGroupInstances[name]
            session.viewports[name].layers[name].assemblyDisplay.displayGroupInstances[name]
            import visualization
            session.viewports[name].layers[name].odbDisplay.displayGroupInstances[name]
            import part
            session.viewports[name].layers[name].partDisplay.displayGroupInstances[name]
            session.viewports[name].odbDisplay.displayGroupInstances[name]
            session.viewports[name].partDisplay.displayGroupInstances[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A Boolean specifying whether the display options stored on the DisplayGroupInstance
    #: object should be synchronized with changes to the viewport display options. This member
    #: is available only for DisplayGroupInstance objects that are members of the
    #: DisplayGroupInstance repository member of the OdbDisplay object. The default value is
    #: OFF.
    lockOptions: Boolean = OFF

    #: An OdbDisplayOptions object specifying this member is available only for
    #: DisplayGroupInstance objects that are members of the DisplayGroupInstance repository
    #: member of the OdbDisplay object.
    odbDisplayOptions: OdbDisplayOptions = OdbDisplayOptions()

    @abaqus_method_doc
    def nodes(self):
        """This method is used to obtain the list of nodes present in the DisplayGroupInstance object. It
        returns a Dictionary object keyed by part instance names, the value of which is a list of user node
        labels belonging to the part instance and contained in the DisplayGroupInstance object. This method is
        available only for DisplayGroupInstance objects that are members of the DisplayGroupInstance repository
        member of OdbDisplay object.

        Returns
        -------
        dict
            A Dictionary object.
        """
        ...

    @abaqus_method_doc
    def elements(self):
        """This method returns the list of elements present in the DisplayGroupInstance object. The elements
        method returns a Dictionary object that uses part instance names for the keys. The value of the items in
        the Dictionary object is a List of user element labels that belong to the part instance and are
        contained in the DisplayGroupInstance object. This method is available only for DisplayGroupInstance
        objects that are members of the DisplayGroupInstance repository member of the OdbDisplay object.

        Returns
        -------
        dict
            A Dictionary object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, lockOptions: Boolean = OFF):
        """This method modifies the DisplayGroupInstance object. The setValues method is available only for
        DisplayGroupInstance objects that are members of the DisplayGroupInstance repository member of the
        OdbDisplay object.

        Parameters
        ----------
        lockOptions
            A Boolean specifying whether the display options stored on the DisplayGroupInstance
            object should be synchronized with changes to the viewport display options. This member
            is available only for DisplayGroupInstance objects that are members of the
            DisplayGroupInstance repository member of the OdbDisplay object. The default value is
            OFF.
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    AUTOMATIC,
    OFF,
    ON,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Area import Area
from .TextStyle import TextStyle


@abaqus_class_doc
class Legend:
    """The Legend object is used to store the display attributes of the chart legend. A legend object is
    automatically created when creating a Chart object.

    .. note::
        This object can be accessed by::

            import visualization
            session.charts[name].legend
            session.defaultChartOptions.legend
            session.xyPlots[name].charts[name].legend
    """

    #: A SymbolicConstant specifying how the minimum and maximum values are formatted. Possible
    #: values are AUTOMATIC, DECIMAL, SCIENTIFIC, and ENGINEERING. The default value is
    #: AUTOMATIC.
    numberFormat: SymbolicConstant = AUTOMATIC

    #: An Int specifying the number of significant digits displayed for the minimum and maximum
    #: values. Possible values are 1 to 7. The default value is 2.
    numDigits: int = 2

    #: A Boolean specifying whether to show the legend. The default value is ON.
    show: Boolean = ON

    #: A Boolean specifying whether to display the minimum and maximum values. The default
    #: value is OFF.
    showMinMax: Boolean = OFF

    #: An Area object specifying the area of the legend.
    area: Area = Area()

    #: A TextStyle object specifying the text properties used to display the legend text.
    textStyle: TextStyle = TextStyle()

    #: A String specifying the title to appear on the legend. The default value is an empty
    #: string.
    title: str = ""

    #: A TextStyle object specifying the text properties used to display the legend title.
    titleStyle: TextStyle = TextStyle()

    @abaqus_method_doc
    def setValues(
        self,
        legend: "Legend",
        show: Boolean = ON,
        showMinMax: Boolean = OFF,
        title: str = "",
        numberFormat: Literal[C.SCIENTIFIC, C.AUTOMATIC, C.ENGINEERING, C.DECIMAL] = AUTOMATIC,
        numDigits: int = 2,
        textStyle: TextStyle | None = None,
        titleStyle: TextStyle | None = None,
    ):
        """This method modifies the Legend object.

        Parameters
        ----------
        legend
            A Legend object from which attributes are to be copied.
        show
            A Boolean specifying whether to show the legend. The default value is ON.
        showMinMax
            A Boolean specifying whether to display the minimum and maximum values. The default
            value is OFF.
        title
            A String specifying the title to appear on the legend. The default value is an empty
            string.
        numberFormat
            A SymbolicConstant specifying how the minimum and maximum values are formatted. Possible
            values are AUTOMATIC, DECIMAL, SCIENTIFIC, and ENGINEERING. The default value is
            AUTOMATIC.
        numDigits
            An Int specifying the number of significant digits displayed for the minimum and maximum
            values. Possible values are 1 to 7. The default value is 2.
        textStyle
            A TextStyle object specifying the text properties used to display the legend text.
        titleStyle
            A TextStyle object specifying the text properties used to display the legend title.
        """
        ...



from __future__ import annotations

from .AFXDataComponent import AFXDataComponent
from .constants import DEFAULT_PAD
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXObject import FXObject
from .FXPacker import FXPacker

#: Use a check button instead of a label.
AFXSPINNER_CHECKBUTTON: int = hash("AFXSPINNER_CHECKBUTTON")

#: Use a radio button instead of a label.
AFXSPINNER_RADIOBUTTON: int = hash("AFXSPINNER_RADIOBUTTON")

#: Orient label or button above spinner.
AFXSPINNER_VERTICAL: int = hash("AFXSPINNER_VERTICAL")

#: Configure spinner to the read-only state.
AFXSPINNER_READONLY: int = hash("AFXSPINNER_READONLY")


class AFXSpinner(FXPacker, AFXDataComponent):
    """This class contains a label that precedes a spin box that allows the user to specify a value by clicking
    on its arrow buttons."""

    #: ID for the check or radio button.
    ID_BUTTON: int = hash("ID_BUTTON")

    #: ID for the spinner.
    ID_SPINNER: int = hash("ID_SPINNER")

    def __init__(
        self,
        p: FXComposite,
        ncols: int,
        labelText: str,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = 0,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_PAD,
        pr: int = DEFAULT_PAD,
        pt: int = DEFAULT_PAD,
        pb: int = DEFAULT_PAD,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        ncols : int
            Number of columns.
        labelText : str
            Label string.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID
        opts : int
            Options and hints.
        x : int
            X coordinate of the origin.
        y : int
            Y coordinate of the origin.
        w : int
            Width of the widget.
        h : int
            Height of the widget.
        pl : int
            Left padding (margin).
        pr : int
            Right padding (margin).
        pt : int
            Top padding (margin).
        pb : int
            Bottom padding (margin).
        """

    def create(self):
        """Creates the spinner.

        Reimplemented from FXComposite.
        """

    def disable(self):
        """Disables the spinner.

        Reimplemented from FXWindow.
        """

    def enable(self):
        """Enables the spinner.

        Reimplemented from FXWindow.
        """

    def getCheck(self):
        """Returns the state of the check button or the radio button."""

    def getHelpText(self):
        """Returns the status line help text."""

    def getIncrement(self):
        """Returns the spinner increment."""

    def getLabelFont(self):
        """Returns the label font."""

    def getLabelText(self):
        """Returns the label string."""

    def getRange(self):
        """Returns a sequence of ints (low, high) representing the widget's allowable minimum and maximum
        values."""

    def getTipText(self):
        """Returns the tool tip message."""

    def getValue(self):
        """Returns the spinner value."""

    def isEditable(self):
        """Returns True if the text in the text field may be edited."""

    def isReadOnlyState(self):
        """Returns True if the spinner appears in the read-only state."""

    def setCheck(self, state: bool):
        """Sets the state of the check button or the radio button.

        Parameters
        ----------
        state : bool
            State.
        """

    def setCheckButtonSelector(self, sel: int):
        """Sets the message ID of the check button or the radio button.

        Parameters
        ----------
        sel : int
            Selector.
        """

    def setCheckButtonTarget(self, tgt: FXObject):
        """Sets the message target of the check button or the radio button.

        Parameters
        ----------
        tgt : FXObject
            Target.
        """

    def setEditable(self, edit: bool = True):
        """Sets the editable state for the input field.

        Parameters
        ----------
        edit : bool
            If True, input field is editable.
        """

    def setHelpText(self, text: str):
        """Sets the status line help text.

        Parameters
        ----------
        text : str
            Help text.
        """

    def setIncrement(self, incr: int):
        """Sets the spinner increment.

        Parameters
        ----------
        incr : int
            Increment.
        """

    def setLabelFont(self, fnt: FXFont):
        """Sets the label font.

        Parameters
        ----------
        fnt : FXFont
            Label font.
        """

    def setLabelText(self, txt: str):
        """Sets the label string.

        Parameters
        ----------
        txt : str
            Label text.
        """

    def setRange(self, low: int, high: int):
        """Sets the spinner range.

        Parameters
        ----------
        low : int
            Minimum value.
        high : int
            Maximum value.
        """

    def setReadOnlyState(self, readonly: bool = True):
        """Sets the read-only state of the spinner.

        Parameters
        ----------
        readonly : bool
            State.
        """

    def setTipText(self, text: str):
        """Sets the tool tip message.

        Parameters
        ----------
        text : str
            Tooltip text.
        """

    def setValue(self, val: int, notify: bool = False):
        """Sets the spinner's value.

        Parameters
        ----------
        val : int
            Value.
        notify : bool
            Notification flag.
        """



from __future__ import annotations

from typing import Any


def execfile(
    filename: str,
    globals: dict[str, Any] | None = None,
    locals: dict[str, Any] | None = None,
) -> None:
    """Read and execute a Python script from a file. The globals and locals are dictionaries, defaulting to the
    current globals and locals.  If only globals is given, locals defaults to it.

    Parameters
    ----------
    filename : os.PathLike
    globals : dict | None, optional, by default None
    locals : dict | None, optional, by default None
    """
    ...


def execPyFile(
    filePath: str,
    globs: dict[str, Any] | None = None,
    locs: dict[str, Any] | None = None,
    atxPort: int = -1,
) -> None:
    """Similar to execfile, but will run a compiled .pyc file
    The default globals and locals must be those of the caller
    Reference:
    http://www.python.org/search/hypermail/python-recent/0186.html
    If atxPort != -1, being used to run Kernel from external tester

    Parameters
    ----------
    filePath : os.PathLike
    globs : dict | None, optional, by default None
    locs : dict | None, optional, by default None
    atxPort : int, optional, by default -1
    """
    ...


def raw_input(prompt: str = "") -> str:
    """Read a string from standard input.  The trailing newline is stripped. If the user hits EOF (Unix: Ctl-D,
    Windows: Ctl-Z+Return), raise EOFError. On Unix, GNU readline is used if enabled.  The prompt string, if
    given, is printed without a trailing newline before reading.

    Parameters
    ----------
    prompt : str, optional
        The prompt written to standard output, by default ''

    Returns
    -------
    str
        The input value converted to a string
    """
    return input(prompt)


def cliCommand(text: str) -> None:
    """Excutes Abaqus/CAE CLI command.

    called from cmdK_CommandDeliveryRole::sendCliCommand

    Parameters
    ----------
    text : str
        Command
    """
    ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import DEFAULT_MODEL, SymbolicConstant
from .Leaf import Leaf


@abaqus_class_doc
class LeafFromSurfaceSets(Leaf):
    """The LeafFromSurfaceSets object can be used whenever a Leaf object is expected as an argument. Leaf
    objects are used to specify the items in a display group. Leaf objects are constructed as temporary objects,
    which are then used as arguments to DisplayGroup commands. The LeafFromSurfaceSets object is derived from
    the Leaf object.

    .. note::
        This object can be accessed by::

            import displayGroupOdbToolset
    """

    #: A SymbolicConstant specifying the leaf type. Possible values are EMPTY_LEAF,
    #: DEFAULT_MODEL, ALL_ELEMENTS, ALL_NODES, and ALL_SURFACES.
    leafType: SymbolicConstant

    #: A sequence of Strings specifying surface sets, or a String specifying a single surface
    #: set.
    surfaceSets: tuple[str, ...] = ()

    @abaqus_method_doc
    def __init__(self, surfaceSets: tuple):
        """This method creates a Leaf object from a sequence of surface sets.

        .. note::
            This function can be accessed by::

                LeafFromSurfaceSets

        Parameters
        ----------
        surfaceSets
            A sequence of Strings specifying surface sets, or a String specifying a single surface
            set.

        Returns
        -------
        LeafFromSurfaceSets
            A LeafFromSurfaceSets object.
        """
        super().__init__(DEFAULT_MODEL)



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class DesignResponse:
    """The DesignResponse object is the abstract base type for other DesignResponse objects. The
    DesponseResponse object has no explicit constructor. The methods and members of the DesignResponse object
    are common to all objects derived from DesignResponse.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].designResponses[name]
    """

    #: A String specifying the design response repository key.
    name: str = ""



from __future__ import annotations

from .constants import DEFAULT_SPACING
from .FXComposite import FXComposite
from .FXObject import FXObject
from .FXPacker import FXPacker

#: Tabs on top (default).
TABBOOK_TOPTABS: int = hash("TABBOOK_TOPTABS")

#: Tabs on bottom.
TABBOOK_BOTTOMTABS: int = hash("TABBOOK_BOTTOMTABS")

#: Tabs on left.
TABBOOK_SIDEWAYS: int = hash("TABBOOK_SIDEWAYS")

#: Tabs on left.
TABBOOK_LEFTTABS: int = hash("TABBOOK_LEFTTABS")

#: Tabs on right.
TABBOOK_RIGHTTABS: int = hash("TABBOOK_RIGHTTABS")

#: Normal tab book style.
TABBOOK_NORMAL: int = hash("TABBOOK_NORMAL")


class FXTabBar(FXPacker):
    """The tab bar layout manager arranges tab items side by side, and raises the active tab item above the
    neighboring tab items.

    The tab bar can be have the tab items on the top or bottom for horizontal arrangement, or on the left or
    right for vertical arrangement.
    """

    #: Sent from one of the FXTabItems.
    ID_OPEN_ITEM: int = hash("ID_OPEN_ITEM")

    #: Switch to panel ID_OPEN_FIRST+i.
    ID_OPEN_FIRST: int = hash("ID_OPEN_FIRST")

    def __init__(
        self,
        p: FXComposite,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = TABBOOK_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_SPACING,
        pr: int = DEFAULT_SPACING,
        pt: int = DEFAULT_SPACING,
        pb: int = DEFAULT_SPACING,
    ):
        """Construct a tab bar.

        Parameters
        ----------
        p : FXComposite

        tgt : FXObject | None

        sel : int

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int
        """

    def create(self):
        """Create all of the server-side resources for this window // CAE.

        Reimplemented from FXComposite.
        """

    def getCurrent(self):
        """Return the currently active tab item."""

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXPacker. Reimplemented in FXTabBook.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXPacker. Reimplemented in FXTabBook.
        """

    def getTabStyle(self):
        """Return tab bar style."""

    def setCurrent(self, panel: int, notify: bool = False):
        """Change currently active tab item; this raises the active tab item slightly above the neighboring tab
        items.

        Parameters
        ----------
        panel : int

        notify : bool
        """

    def setTabStyle(self, style: int):
        """Change tab tab style.

        Parameters
        ----------
        style : int
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, ON, Boolean


@abaqus_class_doc
class GeometryDisplayOptions:
    """The GeometryDisplayOptions object stores settings that specify how assemblies are to be displayed in a
    particular viewport. The GeometryDisplayOptions object has no constructor. When you create a new viewport,
    the settings are copied from the current viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].assemblyDisplay.geometryOptions
            session.viewports[name].layers[name].assemblyDisplay.geometryOptions
            session.viewports[name].layers[name].partDisplay.geometryOptions
            session.viewports[name].partDisplay.geometryOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        geometryEdgesInShaded: Boolean = ON,
        geometryHiddenEdges: Boolean = OFF,
        geometrySilhouetteEdges: Boolean = ON,
        datumAxes: Boolean = ON,
        datumCoordSystems: Boolean = ON,
        datumPlanes: Boolean = ON,
        referencePointLabels: Boolean = ON,
        referencePointSymbols: Boolean = ON,
        referenceRepresentation: Boolean = OFF,
        referenceRepTranslucency: Boolean = ON,
    ):
        """This method modifies the GeometryDisplayOptions object.

        Parameters
        ----------
        geometryEdgesInShaded
            A Boolean specifying whether geometry edges are displayed in shaded mode. The default
            value is ON.
        geometryHiddenEdges
            A Boolean specifying whether geometry hidden edges are displayed (dotted) in hidden line
            mode. The default value is OFF.
        geometrySilhouetteEdges
            A Boolean specifying whether geometry silhouette edges are displayed. The default value
            is ON.
        datumAxes
            A Boolean specifying whether datum axes are shown. The default value is ON.
        datumCoordSystems
            A Boolean specifying whether datum coordinate systems are shown. The default value is
            ON.
        datumPlanes
            A Boolean specifying whether datum planes are shown. The default value is ON.
        referencePointLabels
            A Boolean specifying whether referencePoint labels are shown. The default value is ON.
        referencePointSymbols
            A Boolean specifying whether referencePoint symbols are shown. The default value is ON.
        referenceRepresentation
            A Boolean specifying whether geometry that belongs to the reference representation of
            the Part or Instance is shown. The default value is OFF.
        referenceRepTranslucency
            A Boolean specifying whether to apply translucency to the geometry that belongs to the
            reference representation of the Part or Instance. The default value is ON.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from pathlib import Path
from re import error as RegexError

from .cli import AbqpyCLI, abaqus
from .run import run

try:
    from ._version import version as _default_version
except ImportError:
    _default_version = "2020.0.0-dev"


def _get_version():
    root = Path(__file__).resolve().parents[2]
    if (root / ".git").exists() and not (root / ".git/shallow").exists():
        try:
            import setuptools_scm

            return setuptools_scm.get_version(root=str(root))
        except (ImportError, RegexError, LookupError):
            return _default_version
    else:
        return _default_version


__version__ = _get_version()
__semver__ = __version__.split("+")[0]
version_info = __semver__.split(".")

__all__ = [
    "run",
    "abaqus",
    "AbqpyCLI",
    "version_info",
    "__version__",
    "__semver__",
]



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class AbaqusNDarray:
    """The AbaqusNDarray object is a sequence object derived from numpy.ndarray and is used to store numeric
    keyword data from an Abaqus input file. This object is similar to the numpy.ndarray object, but the numeric
    elements are returned as standard Python objects, not numpy numeric types. The numeric elements can be:

    - All ints.
    - All floats.
    - First column int, all other columns floats.

    In the last of these cases, the member **colZeroIsInt** will be True; in the other two
    cases, it will be False.
    """

    ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class ModelDot:
    """The ModelDot object can be used to access an actual MeshNode, ReferencePoint, or ConstrainedSketchVertex
    object."""

    ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class Ratios:
    """The Ratios object specifies ratios that define anisotropic swelling.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].moistureSwelling.ratios
            mdb.models[name].materials[name].swelling.ratios
            import odbMaterial
            session.odbs[name].materials[name].moistureSwelling.ratios
            session.odbs[name].materials[name].swelling.ratios

        The table data for this object are:

        - r11.
        - r22.
        - r33.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - RATIOS
    """

    #: A sequence of sequences of Floats specifying the items described below.
    table: tuple[tuple[float, ...], ...] = ()

    #: A Boolean specifying whether the data depend on temperature. The default value is OFF.
    temperatureDependency: Boolean = OFF

    #: An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    @abaqus_method_doc
    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a Ratios object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].moistureSwelling.Ratios
                mdb.models[name].materials[name].swelling.Ratios
                session.odbs[name].materials[name].moistureSwelling.Ratios
                session.odbs[name].materials[name].swelling.Ratios

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        Ratios
            A Ratios object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Ratios object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import STEP, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Amplitude import Amplitude


@abaqus_class_doc
class SolutionDependentAmplitude(Amplitude):
    """The SolutionDependentAmplitude object defines a solution-dependent amplitude for superplastic forming
    analysis. The SolutionDependentAmplitude object is derived from the Amplitude object.

    .. note::
        This object can be accessed by::

            import amplitude
            mdb.models[name].amplitudes[name]
            import odbAmplitude
            session.odbs[name].amplitudes[name]

        The corresponding analysis keywords are:

        - AMPLITUDE
    """

    #: A String specifying the repository key.
    name: str

    #: A Float specifying the initial amplitude value. Possible values are those between
    #: **minimum** and **maximum**. The default value is 1.0.
    initial: float = 1

    #: A Float specifying the minimum amplitude value. Possible values are those smaller than
    #: **maximum** and **initial**. The default value is 0.1.
    minimum: float = 0

    #: A Float specifying the maximum amplitude value. Possible values are those larger than
    #: **minimum** and **initial**. The default value is 1000.0.
    maximum: float = 1000

    #: A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
    #: and TOTAL. The default value is STEP.
    timeSpan: SymbolicConstant = STEP

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        initial: float = 1,
        minimum: float = 0,
        maximum: float = 1000,
        timeSpan: Literal[C.STEP, C.TOTAL] = STEP,
    ):
        """This method creates a SolutionDependentAmplitude object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SolutionDependentAmplitude
                session.odbs[name].SolutionDependentAmplitude

        Parameters
        ----------
        name
            A String specifying the repository key.
        initial
            A Float specifying the initial amplitude value. Possible values are those between
            **minimum** and **maximum**. The default value is 1.0.
        minimum
            A Float specifying the minimum amplitude value. Possible values are those smaller than
            **maximum** and **initial**. The default value is 0.1.
        maximum
            A Float specifying the maximum amplitude value. Possible values are those larger than
            **minimum** and **initial**. The default value is 1000.0.
        timeSpan
            A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
            and TOTAL. The default value is STEP.

        Returns
        -------
        SolutionDependentAmplitude
            A SolutionDependentAmplitude object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        initial: float = 1,
        minimum: float = 0,
        maximum: float = 1000,
        timeSpan: Literal[C.STEP, C.TOTAL] = STEP,
    ):
        """This method modifies the SolutionDependentAmplitude object.

        Parameters
        ----------
        initial
            A Float specifying the initial amplitude value. Possible values are those between
            **minimum** and **maximum**. The default value is 1.0.
        minimum
            A Float specifying the minimum amplitude value. Possible values are those smaller than
            **maximum** and **initial**. The default value is 0.1.
        maximum
            A Float specifying the maximum amplitude value. Possible values are those larger than
            **minimum** and **initial**. The default value is 1000.0.
        timeSpan
            A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
            and TOTAL. The default value is STEP.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

import ast
import os
from typing import Any, Literal, Optional

from pydantic import BaseModel


class CompatibleBaseModel(BaseModel):
    def model_dump(self, **kwargs) -> dict[str, Any]:
        try:
            return super().model_dump(**kwargs)
        except AttributeError:
            return super().dict(**kwargs)


class AbaqusCAEConfig(CompatibleBaseModel):
    database: Optional[str] = None
    replay: Optional[str] = None
    recover: Optional[str] = None
    startup: Optional[str] = None
    gui: bool = False
    envstartup: bool = True
    savedOptions: bool = True
    savedGuiPrefs: bool = True
    startupDialog: bool = True
    custom: Optional[str] = None
    guiTester: Optional[str] = None
    guiRecord: Optional[bool] = None


class AbaqusPythonConfig(CompatibleBaseModel):
    sim: Optional[str] = None
    log: Optional[str] = None


class AbaqusConfig(CompatibleBaseModel):
    cae: AbaqusCAEConfig
    python: AbaqusPythonConfig

    debug: bool = False
    skip_abaqus: bool = False
    make_docs: bool = False
    cli_traceback_limit: int = 0
    execution_method: Literal["os", "subprocess"] = "os"


class AbaqusCommandOptions(AbaqusCAEConfig, AbaqusPythonConfig): ...


defaults = ast.literal_eval(os.environ.get("ABAQUS_COMMAND_OPTIONS", str({})))
defaults.update(gui=defaults.get("gui", not defaults.pop("noGUI", True)))
options = AbaqusCommandOptions(**defaults)


trues = ["true", "1", "on", "yes"]
config = AbaqusConfig(
    cae=AbaqusCAEConfig(
        database=os.environ["ABAQUS_CAE_DATABASE"] if "ABAQUS_CAE_DATABASE" in os.environ else options.database,
        replay=os.environ["ABAQUS_CAE_REPLAY"] if "ABAQUS_CAE_REPLAY" in os.environ else options.replay,
        recover=os.environ["ABAQUS_CAE_RECOVER"] if "ABAQUS_CAE_RECOVER" in os.environ else options.recover,
        startup=os.environ["ABAQUS_CAE_STARTUP"] if "ABAQUS_CAE_STARTUP" in os.environ else options.startup,
        gui=os.environ["ABAQUS_CAE_GUI"].lower() in trues if "ABAQUS_CAE_GUI" in os.environ else options.gui,
        envstartup=(
            os.environ["ABAQUS_CAE_ENVSTARTUP"].lower() in trues
            if "ABAQUS_CAE_ENVSTARTUP" in os.environ
            else options.envstartup
        ),
        savedOptions=(
            os.environ["ABAQUS_CAE_SAVEDOPTIONS"].lower() in trues
            if "ABAQUS_CAE_SAVEDOPTIONS" in os.environ
            else options.savedOptions
        ),
        savedGuiPrefs=(
            os.environ["ABAQUS_CAE_SAVEDGUIPREFS"].lower() in trues
            if "ABAQUS_CAE_SAVEDGUIPREFS" in os.environ
            else options.savedGuiPrefs
        ),
        startupDialog=(
            os.environ["ABAQUS_CAE_STARTUPDIALOG"].lower() in trues
            if "ABAQUS_CAE_STARTUPDIALOG" in os.environ
            else options.startupDialog
        ),
        custom=os.environ["ABAQUS_CAE_CUSTOM"] if "ABAQUS_CAE_CUSTOM" in os.environ else options.custom,
        guiTester=os.environ["ABAQUS_CAE_GUITESTER"] if "ABAQUS_CAE_GUITESTER" in os.environ else options.guiTester,
        guiRecord=(
            os.environ["ABAQUS_CAE_GUIRECORD"].lower() in trues
            if "ABAQUS_CAE_GUIRECORD" in os.environ
            else options.guiRecord
        ),
    ),
    python=AbaqusPythonConfig(
        sim=os.environ["ABAQUS_PYTHON_SIM"] if "ABAQUS_PYTHON_SIM" in os.environ else options.sim,
        log=os.environ["ABAQUS_PYTHON_LOG"] if "ABAQUS_PYTHON_LOG" in os.environ else options.log,
    ),
    debug=os.environ.get("ABQPY_DEBUG", "false").lower() in trues,
    skip_abaqus=os.environ.get("ABQPY_SKIP_ABAQUS", "false").lower() in trues,
    make_docs=os.environ.get("ABQPY_MAKE_DOCS", "false").lower() in trues,
    cli_traceback_limit=int(os.environ.get("ABQPY_CLI_TRACEBACK_LIMIT", 0)),
    execution_method=os.environ.get("ABQPY_EXECUTION_METHOD", "os"),
)
